@@ -1,3 +1,10 @@
+class RawMessage:
+    def __init__(self, message):
+        self.message = message
+
+    def __str__(self):
+        return self.message
+
 class Message(object):
     def __init__(self, channel="", sender_nick="", message="", is_private=False, strip=True):
         """Message.
